/*
---------------------------------------------------------------------
Retorna o ID do PIC informado na linha de comando          01/09/2020

Le a tabela do arquivo 'sppp_config.cfg' e obtem o ID referente
ao chip informado na opcao '-pic' da linha de comando, colocando
os 2 caracteres do ID na variavel global 'char picid[]'.

Retorna 0 se encontrou o nome do chip no arquivo 'sppp_config.cfg'.
Nesse caso o ID estara disponivel na variavel 'char picid[]'.

Retorna -1 se nao encontrou o nome do chip no arquivo.
Nesse caso o valor em 'char picid[]' e' irrelevante.

ENTRADA: char *pic --> nome do microcontrolador (algo com PIC12F675)
SAIDA  :  0 --> nome do chip encontrado
         -1 --> nome do chip nao encontrado
---------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
int getPicId(char *pic) {


  /* Obtem memoria para o buffer de leitura de linhas */
  extern char picid[12];                 /* usado por getPicId() */
  char *buffer;
  size_t buffsize = 28;
  /* size_t linha; */
  buffer = (char *)malloc(buffsize * sizeof(char));
  if( buffer == NULL) {
     printf("getPicId(1): NAO CONSEGUIU RESERVAR MEMORIA\n");
     return -1;
  }


  /* Abre o arquivo de configuracao para leitura */
  char *cfgfile = "sppp_config.cfg";
  FILE *config;
  config = fopen(cfgfile,"r");
  if(!config) {
    printf("getPicId(2): ERRO ABRINDO O ARQUIVO \'%s\'\n", cfgfile );
    return -1;
  }


  /* Leitura do arquivo de configuracao */
  char picname[20];
  int j=0;
  int sz;
  int i;
  while (getline(&buffer, &buffsize, config) > 0) {

    if( strstr(buffer, "[micros]") != NULL ) { continue; }  // despreza o nome da sessao
    j=0;
    memset(&picname, '\0', sizeof(picname));
    sz = sizeof(pic) + 4;

    for(i=3; i<sz; i++) {
       picname[j] = buffer[i];
       j++;
    }

    if(strcmp(pic, picname) == 0 ) {
       fclose(config);
       picid[0] = buffer[0];
       picid[1] = buffer[1];
       return 0;
    }

  }

  fclose(config);
  return -1;
}








