/*
 ---------------------------------------------------------------------------
 Mostra, no terminal, informacoes sobre o chip alvo               26/08/2020

 Le a memoria de configuracao simulando o comando -de (opcao 1) e
 carrega 'main_buffer' com os dados necessarios.

 Retorna o codigo (Device ID) do microcontrolador montado no circuito
 programador.

    0x7E - PIC12F675
    0x82 - PIC16F627A
    0x83 - PIC16F628A
    0x88 - PIC16F648A
    0x8C - PIC16F688

 ---------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include "Includes/getchipid.h"

#define _devOffset 12
#define _cfwOffset  14
int runOptionInfo(char* opt) {

    extern int  serial_port;               /* terminal file handler */
    extern char read_buf[64];              /* input buffer for programmer response */
    extern char main_buffer[4096];         /* input buffer to hold lot of data */
    extern unsigned int picdet;            /* microcontrolador detetado */

    char facke[2] = {'1','\r'};         // para obter memoria de configuracao
    unsigned char chipuid[8];
    unsigned char chipdev[2];
    unsigned char chipcfw[2];
    char picname[16];
    int  i;

    printf("\n");
    printf("O SEGUINTE MICROCONTROLADOR ESTA MONTADO\n");

    memset(&read_buf, '\0', sizeof(read_buf));
    memset(&main_buffer, '\0', sizeof(main_buffer));
    write(serial_port, facke, 2);       // obtem memoria de configuracao

    read(serial_port, &main_buffer, sizeof(read_buf));


    printf("DEVICE ID...................... 0x");
    for (i=0; i<2; i++) {
       chipdev[i] = main_buffer[_devOffset+i];
       printf("%1.2x",chipdev[i]);
    }

    picdet = getChipId();   // vai buscar o codigo do microcontrolador

    if(picdet == 0x7E) { strcpy(picname, "PIC12F675"); }
    if(picdet == 0x82) { strcpy(picname, "PIC16F627A"); }
    if(picdet == 0x83) { strcpy(picname, "PIC16F628A"); }
    if(picdet == 0x88) { strcpy(picname, "PIC16F648A"); }
    if(picdet == 0x8C) { strcpy(picname, "PIC16F688"); }

    printf(" [0x%x] %s\n", picdet, picname);

    printf("CONFIGURATION WORD............. 0x");
    for (i=0; i<2; i++) {
       chipcfw[i] = main_buffer[_cfwOffset+i];
       printf("%1.2x",chipcfw[i]);
    }
    printf("\n");


    printf("USER ID........................ 0x");
    for (i=0; i<8; i++) {
       chipuid[i] = main_buffer[i];
       printf("%1.2x",chipuid[i]);
    }
    printf("\n\n");


    return picdet;
}

