/* -----------------------------------------------------------------------------
   GRAVACAO DE PROGRAMA NO MICROCONTROLADOR ALVO                      25/08/2020

   Esta funcao grava o programa no microcontrolador alvo.

   Entrada: char* opt ---> numero da opcao de gravacao para o programador
            char* chip --> tipo de microcontroldor
   -----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include "Includes/ascii2hex.h"

int runOptionWP(char* opt, char* chip) {

    extern int  serial_port;               /* terminal file handler */
    extern char read_buf[64];              /* input buffer for programmer response */
    extern char main_buffer[4096];         /* input buffer to hold lot of data */
    extern char* programfile;              /* nome do arquivo .hex */

       FILE *localhexf;
       char ch;
       int i;
       int ix = 0;
       /* int indx = 0; */
      /* int num_bytes = 0; */
       char EOT[] = "EOT";

       // ----------------------------------------------------------------------
       // Tenta abrir o arquivo de programa
       // ----------------------------------------------------------------------
       printf("runOptionWP(); GRAVANDO \'%s\' NO %s\n", programfile, chip);

       localhexf = fopen(programfile, "r");                                     // open program file
       if (localhexf == NULL) {                                                 // successfully open?
          close(serial_port);                                                   // unfortunatly not, so
          printf("runOptionWP(); ERRO TENTANDO ABRIR \'%s\'\n", programfile);                  // send error message
          return -1;                                                            // end of job
       }                                                                        /* if (localhexf == NULL) */
       else {

          // -------------------------------------------------------------------
          // Inicio da programacao
          // -------------------------------------------------------------------
          write(serial_port, opt, 2);
          printf("runOptionWP(); ENVIANDO \'-wp\'. AGUARDANDO RESPOSTA... ");
          memset(&read_buf, '\0', sizeof(read_buf));                            // cleanup serial port buffer
          read(serial_port, read_buf, sizeof(read_buf));            // wait response 'ACK'
          char *ptr = strstr(read_buf, "ACK");                                  // scan buffer looking for 'ACK' response
          if(ptr == NULL) {                                                     // if not an 'ACK' then
             close(serial_port);                                                // close serial port
             printf("runOptionWP(); PROGRAMA NAO RESPONDE\nVERIFIQUE A CONEXAO\n");
             return -1;                                                         // end of job
          }
          printf("runOptionWP(); ACK\n");
          // -------------------------------------------------------------------
          // Le o arquivo, caracter por caracter, ate o fim.
          // -------------------------------------------------------------------
          /* int nr = 0;   */
          int cr=0;
          memset(&read_buf, '\0', sizeof(read_buf));                            // cleanup the serial port buffer
          memset(&main_buffer, '\0', sizeof(main_buffer));                      // cleanup the main buffer
          while( (ch = fgetc(localhexf)) != EOF ) {                              // get one character from file
             if( (ch == ':') || (ch == '\n')) { continue; }                     // ignore \n (0x0A)
             // ----------------------------------------------------------------
             // Insere um caracter no buffer
             // ----------------------------------------------------------------
             main_buffer[ix] = ch;                                              // fill buffer
             ix++;                                                              // else increment index

             // ----------------------------------------------------------------
             // Envia cada linha de programa ao programador
             // ----------------------------------------------------------------
             if(ch == '\r') {                                                  // if end of a line then
                for(i=0; i<(ix-1); i+=2) {
                   unsigned int cv = (ascii2hex(main_buffer[i])<<4) | ascii2hex(main_buffer[i+1]);
                   write(serial_port, &cv, 1);                                  // send one byte to progrmmer
                }
                write(serial_port, &ch, 1);                                     // send end of the line ('\r')
                cr++;
                printf("Linha %.4d[%.2d] %s\n", cr, (ix-1), main_buffer);
                read(serial_port, read_buf, sizeof(read_buf));      // wait response 'ACK'
                char *ptr1 = strstr(read_buf, "ACK");                           // scan buffer looking for 'ACK' response
                char *ptr2 = strstr(read_buf, "EOT");                           // scan buffer looking for 'EOT' response
                char *ptr3 = strstr(read_buf, "ERC");                           // scan buffer looking for 'ERC' response

                if(ptr1 != NULL) {                                               // if 'ACK' received then
                  memset(&read_buf, '\0', sizeof(read_buf));                    // cleanup the serial port buffer
                  memset(&main_buffer, '\0', sizeof(main_buffer));              // cleanup the main buffer
                  ix=0;                                                         // reset the buffer pointer
                }
                else if(ptr2 != NULL) {
                  printf("\nrunOptionWP(); PROGRAMADOR REPORTOU ERRO NO CRC\n");
                  printf("runOptionWP(); PROGRAMACAO CANCELADA\n\n");
                  close(serial_port);                                           // close serial port
                  return -1;                                                    // end of job
                }
                else if(ptr3 != NULL) {
                  printf("\nrunOptionWP(); ERRO DE GRAVACAO\n");
                  printf("runOptionWP(); PROGRAMACAO CANCELADA\n\n");
                  close(serial_port);                                           // close serial port
                  return -1;                                                    // end of job
                }

                else {
                  printf("\n");
                  printf("runOptionWP(); PROGRAMA NAO RESPONDE\nVERIFIQUE A CONEXAO\n\n");
                  close(serial_port);                                           // close serial port
                  return -1;                                                    // end of job
                }                                                               /* if(ptr != NULL) */
             }                                                                  /* if(ch == '\r')  */
          }                                                                     /* end of --> while((ch = fgetc(localhexf)) != EOF ) */

          // -------------------------------------------------------------------
          // Enviamos 'EOT' para encerrar o programa
          // -------------------------------------------------------------------
          fclose(localhexf);                                                    // close .hex file
          printf("runOptionWP(); ARQUIVO \'%s\' FECHADO\n", programfile);
          write(serial_port, EOT, sizeof(EOT));                                 // send EOT to programmer
          printf("runOptionWP(); ENVIANDO \'EOT\'. AGUARDANDO RESPOSTA... ");
          read(serial_port, read_buf, sizeof(read_buf));            // wait response 'ACK'
          ptr = strstr(read_buf, "ACK");                                        // scan buffer looking for 'ACK' response
          if(ptr == NULL) {                                                     // if not an 'ACK' then
            printf("???\n");
            close(serial_port);
            return -1;                                                          // end of job
          }
          printf("ACK\n");
       }                                                                        /* end of if (localhexf== NULL) */
       close(serial_port);
       return 0;
}                                                                               // fim da funcao runOptionWP()



